@@ -3,19 +3,26 @@
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Check for new / old style related problems."""
+
+from __future__ import annotations
+
 from typing import TYPE_CHECKING
 
 import astroid
 from astroid import nodes
 
 from pylint.checkers import BaseChecker
-from pylint.checkers.utils import check_messages, has_known_bases, node_frame_class
-from pylint.interfaces import IAstroidChecker
+from pylint.checkers.utils import (
+    has_known_bases,
+    node_frame_class,
+    only_required_for_messages,
+)
+from pylint.typing import MessageDefinitionTuple
 
 if TYPE_CHECKING:
     from pylint.lint import PyLinter
 
-MSGS = {
+MSGS: dict[str, MessageDefinitionTuple] = {
     "E1003": (
         "Bad first argument %r given to super()",
         "bad-super-call",
@@ -33,17 +40,14 @@ class NewStyleConflictChecker(BaseChecker):
     * "super" usage
     """
 
-    __implements__ = (IAstroidChecker,)
-
     # configuration section name
     name = "newstyle"
     # messages
     msgs = MSGS
-    priority = -2
     # configuration options
     options = ()
 
-    @check_messages("bad-super-call")
+    @only_required_for_messages("bad-super-call")
     def visit_functiondef(self, node: nodes.FunctionDef) -> None:
         """Check use of super."""
         # ignore actual functions or method within a new style class
@@ -119,5 +123,5 @@ class NewStyleConflictChecker(BaseChecker):
     visit_asyncfunctiondef = visit_functiondef
 
 
-def register(linter: "PyLinter") -> None:
+def register(linter: PyLinter) -> None:
     linter.register_checker(NewStyleConflictChecker(linter))
