@@ -2,19 +2,17 @@
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-from typing import Union
+from __future__ import annotations
 
 import astroid
 from astroid import nodes
 
-from pylint import checkers, interfaces
+from pylint import checkers
 from pylint.checkers import utils
-from pylint.utils.utils import get_global_option
 
 
 class RecommendationChecker(checkers.BaseChecker):
 
-    __implements__ = (interfaces.IAstroidChecker,)
     name = "refactoring"
     msgs = {
         "C0200": (
@@ -65,7 +63,7 @@ class RecommendationChecker(checkers.BaseChecker):
     }
 
     def open(self) -> None:
-        py_version = get_global_option(self, "py-version")
+        py_version = self.linter.config.py_version
         self._py36_plus = py_version >= (3, 6)
 
     @staticmethod
@@ -75,7 +73,9 @@ class RecommendationChecker(checkers.BaseChecker):
             return False
         return utils.is_builtin_object(inferred) and inferred.name == function
 
-    @utils.check_messages("consider-iterating-dictionary", "use-maxsplit-arg")
+    @utils.only_required_for_messages(
+        "consider-iterating-dictionary", "use-maxsplit-arg"
+    )
     def visit_call(self, node: nodes.Call) -> None:
         self._check_consider_iterating_dictionary(node)
         self._check_use_maxsplit_arg(node)
@@ -104,7 +104,9 @@ class RecommendationChecker(checkers.BaseChecker):
             self.add_message("consider-iterating-dictionary", node=node)
 
     def _check_use_maxsplit_arg(self, node: nodes.Call) -> None:
-        """Add message when accessing first or last elements of a str.split() or str.rsplit()."""
+        """Add message when accessing first or last elements of a str.split() or
+        str.rsplit().
+        """
 
         # Check if call is split() or rsplit()
         if not (
@@ -160,7 +162,7 @@ class RecommendationChecker(checkers.BaseChecker):
                 )
                 self.add_message("use-maxsplit-arg", node=node, args=(new_name,))
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "consider-using-enumerate",
         "consider-using-dict-items",
         "use-sequence-for-iteration",
@@ -289,7 +291,7 @@ class RecommendationChecker(checkers.BaseChecker):
                 self.add_message("consider-using-dict-items", node=node)
                 return
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "consider-using-dict-items",
         "use-sequence-for-iteration",
     )
@@ -322,13 +324,13 @@ class RecommendationChecker(checkers.BaseChecker):
                 return
 
     def _check_use_sequence_for_iteration(
-        self, node: Union[nodes.For, nodes.Comprehension]
+        self, node: nodes.For | nodes.Comprehension
     ) -> None:
         """Check if code iterates over an in-place defined set."""
         if isinstance(node.iter, nodes.Set):
             self.add_message("use-sequence-for-iteration", node=node.iter)
 
-    @utils.check_messages("consider-using-f-string")
+    @utils.only_required_for_messages("consider-using-f-string")
     def visit_const(self, node: nodes.Const) -> None:
         if self._py36_plus:
             # f-strings require Python 3.6
@@ -339,7 +341,7 @@ class RecommendationChecker(checkers.BaseChecker):
 
     def _detect_replacable_format_call(self, node: nodes.Const) -> None:
         """Check whether a string is used in a call to format() or '%' and whether it
-        can be replaced by an f-string
+        can be replaced by an f-string.
         """
         if (
             isinstance(node.parent, nodes.Attribute)
