@@ -1,23 +1,35 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
-from typing import Dict, List, Optional, Tuple
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
-from pylint.exceptions import InvalidMessageError, UnknownMessageError
+from __future__ import annotations
+
+from typing import NoReturn
+
+from pylint.exceptions import (
+    DeletedMessageError,
+    InvalidMessageError,
+    UnknownMessageError,
+)
+from pylint.message._deleted_message_ids import is_deleted_msgid, is_deleted_symbol
 
 
 class MessageIdStore:
 
-    """The MessageIdStore store MessageId and make sure that there is a 1-1 relation between msgid and symbol."""
+    """The MessageIdStore store MessageId and make sure that there is a 1-1 relation
+    between msgid and symbol.
+    """
 
-    def __init__(self):
-        self.__msgid_to_symbol: Dict[str, str] = {}
-        self.__symbol_to_msgid: Dict[str, str] = {}
-        self.__old_names: Dict[str, List[str]] = {}
+    def __init__(self) -> None:
+        self.__msgid_to_symbol: dict[str, str] = {}
+        self.__symbol_to_msgid: dict[str, str] = {}
+        self.__old_names: dict[str, list[str]] = {}
+        self.__active_msgids: dict[str, list[str]] = {}
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.__msgid_to_symbol)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         result = "MessageIdStore: [\n"
         for msgid, symbol in self.__msgid_to_symbol.items():
             result += f"  - {msgid} ({symbol})\n"
@@ -26,7 +38,7 @@ class MessageIdStore:
 
     def get_symbol(self, msgid: str) -> str:
         try:
-            return self.__msgid_to_symbol[msgid]
+            return self.__msgid_to_symbol[msgid.upper()]
         except KeyError as e:
             msg = f"'{msgid}' is not stored in the message store."
             raise UnknownMessageError(msg) from e
@@ -39,8 +51,8 @@ class MessageIdStore:
             raise UnknownMessageError(msg) from e
 
     def register_message_definition(
-        self, msgid: str, symbol: str, old_names: List[Tuple[str, str]]
-    ):
+        self, msgid: str, symbol: str, old_names: list[tuple[str, str]]
+    ) -> None:
         self.check_msgid_and_symbol(msgid, symbol)
         self.add_msgid_and_symbol(msgid, symbol)
         for old_msgid, old_symbol in old_names:
@@ -51,7 +63,8 @@ class MessageIdStore:
         """Add valid message id.
 
         There is a little duplication with add_legacy_msgid_and_symbol to avoid a function call,
-        this is called a lot at initialization."""
+        this is called a lot at initialization.
+        """
         self.__msgid_to_symbol[msgid] = symbol
         self.__symbol_to_msgid[symbol] = msgid
 
@@ -61,7 +74,8 @@ class MessageIdStore:
         """Add valid legacy message id.
 
         There is a little duplication with add_msgid_and_symbol to avoid a function call,
-        this is called a lot at initialization."""
+        this is called a lot at initialization.
+        """
         self.__msgid_to_symbol[msgid] = symbol
         self.__symbol_to_msgid[symbol] = msgid
         existing_old_names = self.__old_names.get(msgid, [])
@@ -69,19 +83,19 @@ class MessageIdStore:
         self.__old_names[msgid] = existing_old_names
 
     def check_msgid_and_symbol(self, msgid: str, symbol: str) -> None:
-        existing_msgid: Optional[str] = self.__symbol_to_msgid.get(symbol)
-        existing_symbol: Optional[str] = self.__msgid_to_symbol.get(msgid)
+        existing_msgid: str | None = self.__symbol_to_msgid.get(symbol)
+        existing_symbol: str | None = self.__msgid_to_symbol.get(msgid)
         if existing_symbol is None and existing_msgid is None:
             return  # both symbol and msgid are usable
         if existing_msgid is not None:
             if existing_msgid != msgid:
                 self._raise_duplicate_msgid(symbol, msgid, existing_msgid)
-        if existing_symbol != symbol:
+        if existing_symbol and existing_symbol != symbol:
             # See https://github.com/python/mypy/issues/10559
-            self._raise_duplicate_symbol(msgid, symbol, existing_symbol)  # type: ignore
+            self._raise_duplicate_symbol(msgid, symbol, existing_symbol)
 
     @staticmethod
-    def _raise_duplicate_symbol(msgid: str, symbol: str, other_symbol: str):
+    def _raise_duplicate_symbol(msgid: str, symbol: str, other_symbol: str) -> NoReturn:
         """Raise an error when a symbol is duplicated."""
         symbols = [symbol, other_symbol]
         symbols.sort()
@@ -90,7 +104,7 @@ class MessageIdStore:
         raise InvalidMessageError(error_message)
 
     @staticmethod
-    def _raise_duplicate_msgid(symbol: str, msgid: str, other_msgid: str) -> None:
+    def _raise_duplicate_msgid(symbol: str, msgid: str, other_msgid: str) -> NoReturn:
         """Raise an error when a msgid is duplicated."""
         msgids = [msgid, other_msgid]
         msgids.sort()
@@ -101,17 +115,36 @@ class MessageIdStore:
         )
         raise InvalidMessageError(error_message)
 
-    def get_active_msgids(self, msgid_or_symbol: str) -> List[str]:
-        """Return msgids but the input can be a symbol."""
-        # Only msgid can have a digit as second letter
-        is_msgid: bool = msgid_or_symbol[1:].isdigit()
-        if is_msgid:
+    def get_active_msgids(self, msgid_or_symbol: str) -> list[str]:
+        """Return msgids but the input can be a symbol.
+
+        self.__active_msgids is used to implement a primitive cache for this function.
+        """
+        try:
+            return self.__active_msgids[msgid_or_symbol]
+        except KeyError:
+            pass
+
+        # If we don't have a cached value yet we compute it
+        msgid: str | None
+        deletion_reason = None
+        if msgid_or_symbol[1:].isdigit():
+            # Only msgid can have a digit as second letter
             msgid = msgid_or_symbol.upper()
             symbol = self.__msgid_to_symbol.get(msgid)
+            if not symbol:
+                deletion_reason = is_deleted_msgid(msgid)
         else:
-            msgid = self.__symbol_to_msgid.get(msgid_or_symbol)  # type: ignore
             symbol = msgid_or_symbol
-        if msgid is None or symbol is None or not msgid or not symbol:
+            msgid = self.__symbol_to_msgid.get(msgid_or_symbol)
+            if not msgid:
+                deletion_reason = is_deleted_symbol(symbol)
+        if not msgid or not symbol:
+            if deletion_reason is not None:
+                raise DeletedMessageError(msgid_or_symbol, deletion_reason)
             error_msg = f"No such message id or symbol '{msgid_or_symbol}'."
             raise UnknownMessageError(error_msg)
-        return self.__old_names.get(msgid, [msgid])
+        ids = self.__old_names.get(msgid, [msgid])
+        # Add to cache
+        self.__active_msgids[msgid_or_symbol] = ids
+        return ids
