@@ -2,9 +2,12 @@
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
+from __future__ import annotations
+
 import os
 import sys
-from typing import TYPE_CHECKING, List, Optional, TextIO
+import warnings
+from typing import TYPE_CHECKING, TextIO
 from warnings import warn
 
 from pylint.message import Message
@@ -27,11 +30,17 @@ class BaseReporter:
     name = "base"
     """Name of the reporter."""
 
-    def __init__(self, output: Optional[TextIO] = None) -> None:
-        self.linter: "PyLinter"
+    def __init__(self, output: TextIO | None = None) -> None:
+        if getattr(self, "__implements__", None):
+            warnings.warn(
+                "Using the __implements__ inheritance pattern for BaseReporter is no "
+                "longer supported. Child classes should only inherit BaseReporter",
+                DeprecationWarning,
+            )
+        self.linter: PyLinter
         self.section = 0
         self.out: TextIO = output or sys.stdout
-        self.messages: List[Message] = []
+        self.messages: list[Message] = []
         # Build the path prefix to strip to get relative paths
         self.path_strip_prefix = os.getcwd() + os.sep
 
@@ -39,10 +48,9 @@ class BaseReporter:
         """Handle a new message triggered on the current file."""
         self.messages.append(msg)
 
-    def set_output(self, output: Optional[TextIO] = None) -> None:
+    def set_output(self, output: TextIO | None = None) -> None:
         """Set output stream."""
-        # pylint: disable-next=fixme
-        # TODO: Remove this method after depreciation
+        # TODO: 3.0: Remove deprecated method
         warn(
             "'set_output' will be removed in 3.0, please use 'reporter.out = stream' instead",
             DeprecationWarning,
@@ -53,7 +61,7 @@ class BaseReporter:
         """Write a line in the output buffer."""
         print(string, file=self.out)
 
-    def display_reports(self, layout: "Section") -> None:
+    def display_reports(self, layout: Section) -> None:
         """Display results encapsulated in the layout tree."""
         self.section = 0
         if layout.report_id:
@@ -63,11 +71,11 @@ class BaseReporter:
                 raise ValueError(f"Incorrect child for {layout.children[0].children}")
         self._display(layout)
 
-    def _display(self, layout: "Section") -> None:
+    def _display(self, layout: Section) -> None:
         """Display the layout."""
         raise NotImplementedError()
 
-    def display_messages(self, layout: Optional["Section"]) -> None:
+    def display_messages(self, layout: Section | None) -> None:
         """Hook for displaying the messages of the reporter.
 
         This will be called whenever the underlying messages
@@ -80,12 +88,12 @@ class BaseReporter:
 
     # Event callbacks
 
-    def on_set_current_module(self, module: str, filepath: Optional[str]) -> None:
+    def on_set_current_module(self, module: str, filepath: str | None) -> None:
         """Hook called when a module starts to be analysed."""
 
     def on_close(
         self,
         stats: LinterStats,
-        previous_stats: LinterStats,
+        previous_stats: LinterStats | None,
     ) -> None:
         """Hook called when a module finished analyzing."""
